package com.zhentao.service.impl;

import cn.hutool.core.util.IdUtil;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.sun.xml.internal.bind.v2.model.core.ID;
import com.zhentao.controller.orderinfo.dto.OrderDto;
import com.zhentao.controller.orderinfo.dto.ProductOrder;
import com.zhentao.domain.Coupon;
import com.zhentao.domain.OrderInfo;
import com.zhentao.domain.OrderItem;
import com.zhentao.domain.Productsku;
import com.zhentao.mapper.ProductskuMapper;
import com.zhentao.service.CouponService;
import com.zhentao.service.OrderInfoService;
import com.zhentao.mapper.OrderInfoMapper;
import com.zhentao.service.OrderItemService;
import com.zhentao.service.ProductskuService;
import com.zhentao.tool.TokenUtils;
import com.zhentao.vo.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.List;
import java.util.stream.Collectors;

/**
* @author 86183
* @description 针对表【order_info】的数据库操作Service实现
* @createDate 2025-05-10 10:21:48
*/
@Service
public class OrderInfoServiceImpl extends ServiceImpl<OrderInfoMapper, OrderInfo>
    implements OrderInfoService{
    @Autowired
    private StringRedisTemplate stringRedisTemplate;
    @Autowired
    private ProductskuMapper productskuMapper;
    @Autowired
    private CouponService service;
    @Autowired
    private OrderItemService orderItemService;
    @Transactional
    @Override
    public Result add(OrderDto orderDto) {
        OrderInfo orderInfo = new OrderInfo();
//        设置主键id
        long l = IdUtil.getSnowflake(1, 1).nextId();
        orderInfo.setId(l);
//        设置用户ID
        String userIdFromToken = TokenUtils.getUserIdFromToken(orderDto.getToken());
        if (userIdFromToken==null){
            String token = stringRedisTemplate.opsForValue().get("token");
            String userIdFromToken1 = TokenUtils.getUserIdFromToken(token);
            orderInfo.setUserId(Long.valueOf(userIdFromToken1));
        }else {
            orderInfo.setUserId(Long.valueOf(userIdFromToken));
        }
//        计算总价格
        BigDecimal bigDecimal = new BigDecimal("0");
//        获取下单的所有skuid
        List<Long> collect = orderDto.getList().stream().map(ProductOrder::getSkuId).collect(Collectors.toList());
        List<Productsku> productskus = productskuMapper.selectBatchIds(collect);
        System.err.println(productskus);
        for (Productsku p:productskus) {
            for (ProductOrder a:orderDto.getList()) {
                if (p.getSkuId().equals(a.getSkuId())){
                    bigDecimal = bigDecimal.add(p.getProductPrice().multiply(a.getProductStock()));
                }
            }
        }
        System.err.println(bigDecimal);
        orderInfo.setOrderSum(bigDecimal);
//        设置实际支付金额
        Coupon byId = service.getById(orderDto.getCouid());
        orderInfo.setCouponId(orderDto.getCouid());
        orderInfo.setOrderZsum(bigDecimal.subtract(byId.getCouponMublit()));
        this.save(orderInfo);
//        添加详情
        for (ProductOrder p:orderDto.getList()) {
            OrderItem orderItem = new OrderItem();
            long l1 = IdUtil.getSnowflake(1, 1).nextId();
            orderItem.setId(l1);
            orderItem.setUserId(Long.valueOf(userIdFromToken));
            orderItem.setProductSkuId(l);
            orderItem.setPrice(p.getProductPrice());
            orderItem.setSum(p.getProductStock());
            orderItem.setOrderSum(p.getProductPrice().multiply(p.getProductStock()));
            orderItemService.save(orderItem);
        }
        for (Productsku p:productskus){
            for (ProductOrder a:orderDto.getList()) {
                if (p.getSkuId().equals(a.getSkuId())){
                    p.setProductStock(p.getProductStock().subtract(a.getProductStock()));
                    productskuMapper.updateById(p);
                }
            }
        }
        return Result.OK("添加成功",null);
    }
}




