package com.zhentao.service.impl;

import cn.hutool.core.util.IdUtil;
import cn.hutool.crypto.digest.DigestUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zhentao.controller.user.dto.UserDto;
import com.zhentao.domain.UserLogin;
import com.zhentao.service.UserLoginService;
import com.zhentao.mapper.UserLoginMapper;
import com.zhentao.tool.TokenUtils;
import com.zhentao.vo.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.DigestUtils;

import java.util.Date;
import java.util.UUID;

/**
* @author 86183
* @description 针对表【user_login】的数据库操作Service实现
* @createDate 2025-05-10 10:21:49
*/
@Service
public class UserLoginServiceImpl extends ServiceImpl<UserLoginMapper, UserLogin>
    implements UserLoginService{
    @Autowired
    private StringRedisTemplate stringRedisTemplate;
//注册
    @Override
    public Result register(UserDto userDto) {
//        创建一个用户对象
        UserLogin user = new UserLogin();
        user.setUserUsername(userDto.getUsername());
//        生成一个雪花算法id
        long l = IdUtil.getSnowflake(1, 1).nextId();
        user.setId(l);
//        用UUID生成一个随机字符串
        String uuid = String.valueOf(UUID.randomUUID());
        user.setUserSalt(uuid);
//        对Miami进行MD5加密
        String s = DigestUtils.md5DigestAsHex((uuid+userDto.getPassword()).getBytes());
        user.setUserPassword(s);
        user.setUserCreateTime(new Date());
        user.setUserUpdateTime(new Date());
        boolean save = this.save(user);
        if (save){
            return Result.OK("注册成功",save);
        }
        return Result.ERR("注册失败",null);
    }
//登录
    @Override
    public Result login(UserDto userDto) {
//        查询用户名称是否存在
        QueryWrapper<UserLogin> queryWrapper =new QueryWrapper<>();
        queryWrapper.eq("user_username",userDto.getUsername());
        UserLogin one = this.getOne(queryWrapper);
        if (one==null){
            return Result.ERR("用户名不正确",null);
        }
        String userSalt = one.getUserSalt();
        String s = DigestUtils.md5DigestAsHex((userSalt + userDto.getPassword()).getBytes());
        if (!s.equals(one.getUserPassword())){
            return Result.ERR("密码不正确",null);
        }
        String token = TokenUtils.generateToken(one.getId()+"");
        stringRedisTemplate.opsForValue().set("token",token);
        return Result.OK("登录成功",token);
    }
}




